/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include "param.h"
#include "error.h"
#include "gen.h"

static char     logfilename[MAX_PATH];  /* Filename of logfile */
int             logging_flag=0;         /* 1=logging, 0=no logging */

/*============================================================================
 Start logging
 [INPUT]
  filename      : Filename of logfile
 [RETURN]
  none
  ============================================================================
*/
void start_logging(char *filename)
{
    strncpy(logfilename,filename,sizeof(logfilename)-1);
    logfilename[sizeof(logfilename)-1]='\0';
    logging_flag=1;
}

/*============================================================================
 Terminate logging
 [INPUT]
  none
 [RETURN]
  none
  ============================================================================
*/
void end_logging(void)
{
    logging_flag=0;
}

/*============================================================================
  Display log and store to logfile
 [INPUT]
  error_type    : ERROR_WARNING=Warning, ERROR_FATAL=Fatal Error
  format        : Format string
 [RETURN]
  return value of vprintf()
  ============================================================================
*/
int errorlog(int error_type,const char *format,...)
{
    va_list         args;
    int             r;
    time_t          timeval;
    struct tm       *timep=NULL;
    char            *timesp=NULL;
    char            *c;
    static char     fmt_ext[512];
    static char     datestr[512];
    FILE            *fp;

    /* store error type string to fmt_ext */
    switch(error_type){
        case ERRLOG_NOTIFY: strcpy(fmt_ext,"gdd/NOTIFY ");
                            break;
        case ERRLOG_WARNING:strcpy(fmt_ext,"gdd/WARNING");
                            break;
        case ERRLOG_FATAL:  strcpy(fmt_ext,"gdd/ERROR  ");
                            break;
        default:            strcpy(fmt_ext,"gdd/UNKNOWN");
    };

    /* store date&time to datestr */
    timeval=time(NULL);
    if (timeval!=(time_t)-1){
        timep=localtime(&timeval);
            if (timep!=NULL)
                timesp=asctime(timep);
    }
    if (timesp==NULL)
        strcpy(datestr,"time() error ");
    else{
        strncpy(datestr,timesp,sizeof(datestr)-1);
        datestr[sizeof(datestr)-1]='\0';
        if ((c=strchr(datestr,'\r'))!=NULL
         || (c=strchr(datestr,'\n'))!=NULL) *c='\0';
    }

    /* display log */
    fprintf(stderr,"%s ",datestr);
    fprintf(stderr,"%s ",fmt_ext);
    va_start(args, format);
    r=vfprintf(stderr,format,args);
    va_end(args);

#ifdef WIN32
    {
        static char cbuf[512];

        console_write(datestr);
        console_write(" ");
        console_write(fmt_ext);
        console_write(" ");
        va_start(args, format);
        _vsnprintf(cbuf,sizeof(cbuf),format,args);
        va_end(args);
		console_write(cbuf);
		console_write("\r\n");
    }
#endif

    /* Logging to logfile */
    if (logging_flag && strlen(logfilename)>0){
        r=setperm_logfile(0,logfilename);
        if (r==PERM_NOERROR  || r==PERM_NOTFOUND){
            if ((fp=fopen(logfilename,"a"))!=NULL){
                fprintf(fp,"%s ",datestr);
                fprintf(fp,"%s ",fmt_ext);
                va_start(args, format);
                vfprintf(fp,format,args);
                va_end(args);
                fclose(fp);
                if (r==PERM_NOTFOUND)
                    setperm_logfile(0,logfilename);
            }
        }
    }
    return(r);
}
