/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

 *****************************************************************************/

#include <stdio.h>
#include "depend.h"
#include "param.h"
#include "promisc.h"
#include "analyze.h"
#include "gddconfig.h"
#include "error.h"
#include "gui_win32_console.h"
#include "gui_win32_msghdlr_main.h"
#include "gui_win32_settingwndproc.h"
#include "gdd_printf.h"
#include "resource.h"

/*============================================================================
 WM_INITDIALOG Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_InitDialog(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{
    void PrepareMainDlg(GDDCONFIG *gddc);

    console_initialize(gddc->egui.hWnd);
    PrepareMainDlg(gddc);
    EnableWindow(GetDlgItem(gddc->egui.hWnd,IDC_STOP),FALSE);
    gdd_printf(GDD_BANNER,GDD_VERSION);
    gdd_printf("\nPush \"Go\" button to start capture\n\n");
    return(MH_SUCCESS);
}
/* Prepare contents of dialog box
*/
void PrepareMainDlg(GDDCONFIG *gddc)
{
    int getniclist(char *niclist,unsigned int szniclist);

    HWND            hWnd=gddc->egui.hWnd;
    HINSTANCE       hInst=gddc->egui.hInstance;
    LV_COLUMN       col;
    UINT            i;
    static HWND     hstr1,hstr2;
    static struct{
        char    *capt;
        int     width;
    }stCaption[]={
        {"Date/Time",   148},
        {"SrcIP",   92},
        {"SrcPort", 52},
        {"DstIP",   92},
        {"DstPort", 52},
        {"Type",    48},
        {NULL,      0}
    };

    //** Init Main window

    SendDlgItemMessage(hWnd,IDC_GO,BM_SETIMAGE,IMAGE_ICON,
                       (LPARAM)LoadIcon(hInst,MAKEINTRESOURCE(IDI_GO)));
    SendDlgItemMessage(hWnd,IDC_STOP,BM_SETIMAGE,IMAGE_ICON,
                       (LPARAM)LoadIcon(hInst,MAKEINTRESOURCE(IDI_STOP)));
    SendDlgItemMessage(hWnd,IDC_CFG,BM_SETIMAGE,IMAGE_ICON,
                       (LPARAM)LoadIcon(hInst,MAKEINTRESOURCE(IDI_CFG)));

    //** Init List window

    // Log list
    gddc->egui.hList = CreateWindowEx( 0, WC_LISTVIEW, "",WS_CHILD |
                WS_VISIBLE | LVS_REPORT | WS_BORDER,
                11, 9, 504, 293,gddc->egui.hWnd,(HMENU)1,gddc->egui.hInstance,NULL);

    ListView_SetExtendedListViewStyle(gddc->egui.hList,
        LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES );

    for (i=0;;i++){
        if (stCaption[i].capt==NULL) break;
        col.mask     = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
        col.fmt      = LVCFMT_LEFT;
        col.pszText  = stCaption[i].capt;
        col.cx       = stCaption[i].width;
        col.iSubItem = i;
        ListView_InsertColumn( gddc->egui.hList, i, &col );
    }

    gddc->egui.nList=0;
    ShowWindow(gddc->egui.hList,SW_HIDE);
}

/*============================================================================
 WM_ASYNC_SELECT Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_AsyncSelect(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{
    static unsigned char    buf[CHUNKSIZE];
    unsigned long           len;
    WSABUF                  wsaBuffers;
    DWORD                   Flags;

    wsaBuffers.buf=buf;
    wsaBuffers.len=CHUNKSIZE;
    memset(wsaBuffers.buf,0,CHUNKSIZE);
    Flags=0;
    if(WSARecv(gddc->egui.sock,&wsaBuffers,1,&len,&Flags,NULL,NULL)==SOCKET_ERROR){
        errorlog(ERRLOG_FATAL,"WSARecv error\n");
        return(MH_RECVERR);
    }
    if (packet_analysis(gddc,buf,len)){
        release_connection_table();
        WSACleanup();
        return(MH_ANALYSISERR);
    }
    return(MH_SUCCESS);
}

/*============================================================================
 WM_TIMER Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_Timer(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{
    if (wParam==CONSOLE_TIMERID){
        KillTimer(gddc->egui.hWnd,CONSOLE_TIMERID);
        console_show();
    }else if (wParam == LED2_TIMERID){
        KillTimer(gddc->egui.hWnd,LED2_TIMERID);
        Win32MsgSubfunc_LED(gddc,LED2,LED_OFF,-1);
    }
    return(MH_SUCCESS);
}

/*============================================================================
 WM_COMMAND/IDC_GO Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_CmdGo(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{
    SOCKET  sock;
    HWND    hWnd=gddc->egui.hWnd;

    if (init_connection_table()){
        errorlog(ERRLOG_FATAL,"Can not initialize contable\n");
        return(MH_INITERR);
    }
    if ((sock=setnic_promisc(gddc->nit_dev,gddc->nic_name,NULL))<0){
        errorlog(ERRLOG_FATAL,"setnic_promisc error (%d). \n",sock);
        gdd_printf("You don't have Administrator privilege or captureable\n"); 
        gdd_printf("network interface is not installed on your system\n");
        release_connection_table();
        WSACleanup();
        return(MH_INITERR);
    }
    if (WSAAsyncSelect(sock,hWnd,WM_ASYNC_SELECT,FD_READ)==SOCKET_ERROR){
        errorlog(ERRLOG_FATAL,"WSAAsyncSelect error\n",WSAGetLastError());
        release_connection_table();
        WSACleanup();
        return(MH_INITERR);
    }
    gddc->egui.sock=sock;
    return(MH_SUCCESS);
}

/*============================================================================
 WM_COMMAND/IDC_STOP Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_CmdStop(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{
    SOCKET  sock=gddc->egui.sock;
    HWND    hWnd=gddc->egui.hWnd;

    closesocket(sock);
    WSACleanup();
    return(MH_SUCCESS);
}

/*============================================================================
 WM_COMMAND/IDC_IDC_CFG Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_CmdCfg(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{
    DLGPROC     pProc;
    HINSTANCE   hInst=gddc->egui.hInstance;

    pProc=(DLGPROC)MakeProcInstance(SettingDlgProc,hInst);
    if (pProc!=NULL){
        DialogBoxParam(hInst,MAKEINTRESOURCE(IDD_SETTING),
            gddc->egui.hWnd,pProc,(LPARAM)gddc);
        FreeProcInstance((FARPROC)pProc);
    }

    return(MH_SUCCESS);
}

/*============================================================================
 WM_COMMAND/IDC_HOLDTMPLOG Message Handler
 [INPUT]
  gddc      : Configuration data of gdd
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
MH_RESULT Win32MsgHdlr_CmdHoldTmpLog(GDDCONFIG *gddc,WPARAM wParam,LPARAM lParam)
{

    if (wParam){
        ShowWindow(GetDlgItem(gddc->egui.hWnd,IDC_CONSOLE),SW_HIDE);
        ShowWindow(gddc->egui.hList,SW_SHOW);
        gddc->egui.console_mode=CNSLMD_LIST;
    }else{
        ShowWindow(gddc->egui.hList,SW_HIDE);
        ShowWindow(GetDlgItem(gddc->egui.hWnd,IDC_CONSOLE),SW_SHOW);
        gddc->egui.console_mode=CNSLMD_NORMAL;
    }
    return(MH_SUCCESS);
}

/*============================================================================
 Add to connection list
 [INPUT]
  gddc      : Configuration data of gdd
  datetime  : Date & time
  time      : Time
  sourceIP  : Source IP address
  sourcePort: Source port number
  destIP    : dest IP address
  destPort  : dest port number
  logype    : Type of log (LOGTYPE_BIN to LOGTYPE_NONFILER)
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
int Win32MsgSubfunc_AddList(GDDCONFIG *gddc,char *datetime,
            char *sourceIP,int sourcePort,char *destIP,int destPort,
            int logtype)
{
    LV_ITEM         item;
    char            buf[256];
    static char     *logtypestr[]={STR_LOGTYPE_BIN,STR_LOGTYPE_TEXT,
                                   STR_LOGTYPE_TAGTEXT,STR_LOGTYPE_HTML,
                                   STR_LOGTYPE_NONFILTER};

    item.mask     = LVIF_TEXT;
    item.pszText  = (LPTSTR)datetime;
    item.iItem    = gddc->egui.nList;
    item.iSubItem = 0;
    ListView_InsertItem(gddc->egui.hList, &item );

    item.mask     = LVIF_TEXT;
    item.pszText  = (LPTSTR)sourceIP;
    item.iSubItem = 1;
    ListView_SetItem(gddc->egui.hList, &item );

    sprintf(buf,"%d",sourcePort);
    item.mask     = LVIF_TEXT;
    item.pszText  = (LPTSTR)buf;
    item.iSubItem = 2;
    ListView_SetItem(gddc->egui.hList, &item );

    item.mask     = LVIF_TEXT;
    item.pszText  = (LPTSTR)destIP;
    item.iSubItem = 3;
    ListView_SetItem(gddc->egui.hList, &item );

    sprintf(buf,"%d",destPort);
    item.mask     = LVIF_TEXT;
    item.pszText  = (LPTSTR)buf;
    item.iSubItem = 4;
    ListView_SetItem(gddc->egui.hList, &item );

    item.mask     = LVIF_TEXT;
    item.pszText  = (LPTSTR)logtypestr[logtype];
    item.iSubItem = 5;
    ListView_SetItem(gddc->egui.hList, &item );

    gddc->egui.nList++;

    SendMessage(gddc->egui.hList,WM_VSCROLL,(WPARAM)MAKEWORD(SB_BOTTOM,0),0);
    return(0);
}

/*============================================================================
 Select connection list
 [INPUT]
  gddc      : Configuration data of gdd
  nItem     : Number of Item
 [RETURN]
   0 Success
  <0 Error
  ============================================================================
*/
int Win32MsgSubfunc_SelectList(GDDCONFIG *gddc,int nItem)
{
    char    datetime[256],sourceIP[256],sourcePort[256];
    char    destIP[256],destPort[256],logtype[64];
    char    logfile[MAX_PATH],session_file[MAX_PATH],work_file[MAX_PATH];
    char    extension[32];
    UINT    i;

    ListView_GetItemText(gddc->egui.hList,nItem,0,datetime,sizeof(datetime)-1);
    ListView_GetItemText(gddc->egui.hList,nItem,1,sourceIP,sizeof(sourceIP)-1);
    ListView_GetItemText(gddc->egui.hList,nItem,2,sourcePort,sizeof(sourcePort)-1);
    ListView_GetItemText(gddc->egui.hList,nItem,3,destIP,sizeof(destIP)-1);
    ListView_GetItemText(gddc->egui.hList,nItem,4,destPort,sizeof(destPort)-1);
    ListView_GetItemText(gddc->egui.hList,nItem,5,logtype,sizeof(logtype)-1);

    for (i=0;i<strlen(datetime);i++){
        if (datetime[i]==':') datetime[i]='-';
        else if (datetime[i]==' ') datetime[i]='_';
    }

    if (!strcmp(logtype,STR_LOGTYPE_BIN))
        strcpy(extension,EXT_LOGTYPE_BIN);
    else if (!strcmp(logtype,STR_LOGTYPE_TEXT))
        strcpy(extension,EXT_LOGTYPE_TEXT);
    else if (!strcmp(logtype,STR_LOGTYPE_TAGTEXT))
        strcpy(extension,EXT_LOGTYPE_TAGTEXT);
    else if (!strcmp(logtype,STR_LOGTYPE_HTML))
        strcpy(extension,EXT_LOGTYPE_HTML);
    else if (!strcmp(logtype,STR_LOGTYPE_NONFILTER))
        strcpy(extension,EXT_LOGTYPE_NONFILTER);
    else
        strcpy(extension,EXT_LOGTYPE_TEXT);

    _snprintf(logfile,MAX_PATH,"%s_%s(%s)-%s(%s)",datetime,sourceIP,sourcePort,destIP,destPort);
    _snprintf(session_file,MAX_PATH,"%s\\%s",gddc->templogdir,logfile);
    _snprintf(work_file,MAX_PATH,"%s\\%s.%s",WORKDIR,logfile,extension);

    CopyFile(session_file,work_file,FALSE);
    ShellExecute(gddc->egui.hWnd,"open",work_file,NULL,NULL,SW_SHOWNORMAL);

    return(0);
}

/*============================================================================
 LED ON/OFF
 [INPUT]
  gddc      : Configuration data of gdd
  num       : LED number
  ledsw     : 1=ON,0=OFF
  timeout   : Timeout value (-1=non timeout mode)
 [RETURN]
  none
  ============================================================================
*/
int Win32MsgSubfunc_LED(GDDCONFIG *gddc,int num,int ledsw,int timeout)
{
    HINSTANCE       hInst=gddc->egui.hInstance;
    static HANDLE   hi[2][2];
    DWORD           timerID;
    int             nDlgItem,hn;
    static int      ledswstate[2];

    switch(num){
        case LED1:  nDlgItem=IDC_LED1;
                    timerID=LED1_TIMERID;
                    break;
        case LED2:  nDlgItem=IDC_LED2;
                    timerID=LED2_TIMERID;
                    break;
    }
    switch(ledsw){
        case LED_QUERY:
                if (num==LED1 || num==LED2)
                    return(ledswstate[num]);
                else
                    return(-1);
        case LED_CLEAR:
                if (num==LED1 || num==LED2){
                    ledswstate[num]=-1;
                    return(0);
                }else
                    return(-1);
        case LED_INIT:
                ledswstate[0]=ledswstate[1]=0;
                hi[LED1][LED_OFF]=LoadBitmap(hInst,MAKEINTRESOURCE(IDB_R_LEDOFF));
                hi[LED1][LED_ON]=LoadBitmap(hInst,MAKEINTRESOURCE(IDB_R_LEDON));
                hi[LED2][LED_OFF]=LoadBitmap(hInst,MAKEINTRESOURCE(IDB_G_LEDOFF));
                hi[LED2][LED_ON]=LoadBitmap(hInst,MAKEINTRESOURCE(IDB_G_LEDON));
                if (hi[LED1][LED_OFF]==NULL || hi[LED1][LED_ON]==NULL
                 || hi[LED2][LED_OFF]==NULL || hi[LED2][LED_ON]==NULL)
                    return(-1);
                return(0);
        case LED_TERM:
                DeleteObject(hi[LED1][LED_OFF]);
                DeleteObject(hi[LED1][LED_ON]);
                DeleteObject(hi[LED2][LED_OFF]);
                DeleteObject(hi[LED2][LED_ON]);
                return(0);
        case LED_OFF:
        case LED_ON:
                break;
        default:
                return(-1);
    }


    hn=(ledsw==LED_OFF)?0:1;    
    SendDlgItemMessage(gddc->egui.hWnd,nDlgItem,STM_SETIMAGE,
                       IMAGE_BITMAP,(LPARAM)hi[num][hn]);

    ledswstate[num]=ledsw;

    if (timeout!=LEDTIMER_INF)
        SetTimer(gddc->egui.hWnd,timerID,timeout,NULL);
    return(0);
}
/*============================================================================
 Set paramters to main dialog box
 [INPUT]
  gddc      : Configuration data of gdd
 [RETURN]
  number of interfaces
  ============================================================================
*/
int Win32MsgSubfunc_SetParamsToMainDlg(GDDCONFIG *gddc)
{
    HWND        hWnd=gddc->egui.hWnd;
    HINSTANCE   hInst=gddc->egui.hInstance;
    char        buf[4096],tmpbuf[512],pt[2],*p,*q;
    int         i,r;

    /* Set Source/Dest IP address */
    SetDlgItemText(hWnd,IDC_SRCIP,gddc->sourceIP);
    SetDlgItemText(hWnd,IDC_DSTIP,gddc->destIP);

    /* Set Ports */
    strcpy(buf,"");
    for (i=0;;i++){
        if (gddc->port[i].pst==ENDOFPORT) break;
        
        switch(gddc->port[i].logtype){
            case LOGTYPE_BIN:      sprintf(pt,"%c",C_LOGTYPE_BIN); break;
            case LOGTYPE_TEXT:     strcpy(pt,""); break;
            case LOGTYPE_TAGTEXT:  sprintf(pt,"%c",C_LOGTYPE_TAGTEXT); break;
            case LOGTYPE_HTML:     sprintf(pt,"%c",C_LOGTYPE_HTML); break;
            case LOGTYPE_NONFILTER:sprintf(pt,"%c",C_LOGTYPE_NONFILTER); break;
            default:               strcpy(pt,""); break;
        }
        if (gddc->port[i].pst==gddc->port[i].ped)
            sprintf(tmpbuf,"%s%d ",pt,gddc->port[i].pst);
        else
            sprintf(tmpbuf,"%s%d-%d ",pt,gddc->port[i].pst,gddc->port[i].ped);

        if (strlen(buf)+strlen(tmpbuf) >= sizeof(buf)) break;
        strcat(buf,tmpbuf);
    }
    if (buf[strlen(buf)-1]==' ') buf[strlen(buf)-1]='\0';
    SetDlgItemText(hWnd,IDC_PORT,buf);

    /* Set Check boxes */
    CheckDlgButton(hWnd,IDC_HOLDTMPLOG,gddc->hold);
    CheckDlgButton(hWnd,IDC_PKTDUMP,gddc->mode&MODE_DUMP);
    CheckDlgButton(hWnd,IDC_SECUREMODE,gddc->mode&MODE_SECURE);

    /* Set NIC list */
    SendMessage(GetDlgItem(hWnd,IDC_INTERFACE),CB_RESETCONTENT,0,0);
    if (getniclist(buf,sizeof(buf))<0) return(0);
    for (p=buf,r=i=0;;i++){
        if (!(q=strchr(p,','))) break;
        *q='\0';
        SendMessage(GetDlgItem(hWnd,IDC_INTERFACE),CB_ADDSTRING,0,(LPARAM)p);
        if (!strcmp(p,gddc->nic_name)) r=i;
        p=q+1;
    }
    SendMessage(GetDlgItem(hWnd,IDC_INTERFACE),CB_SETCURSEL,r,0);
    return(i);
}

/*============================================================================
 Get paramters from main dialog box
 [INPUT]
  gddc      : Configuration data of gdd
  paramdisp_flag : if true, parameters will display on console
 [RETURN]
  -1 = Error
   0 = Success
  ============================================================================
*/
int Win32MsgSubfunc_GetParamsFromMainDlg(GDDCONFIG *gddc,int paramdisp_flag)
{
    int         paramcheck(GDDCONFIG *gddc);
    void        paramdisplay_console(GDDCONFIG *gddc);
    int         parse_opt_port(GDDCONFIG *gddc,char *optarg,int *portn);

    HWND        hWnd=gddc->egui.hWnd;
    HINSTANCE   hInst=gddc->egui.hInstance;
    char        buf[4096],*p,*q;
    int         i,portn,r;

    /* Get Source/Dest IP address */
    GetDlgItemText(hWnd,IDC_SRCIP,gddc->sourceIP,MAXLEN_IPADDR);
    GetDlgItemText(hWnd,IDC_DSTIP,gddc->destIP,MAXLEN_IPADDR);
    switch(paramcheck(gddc)){
        case 0:     break;
        case -1:    errorlog(ERRLOG_FATAL,"Invalid SRC/DST IP address\n");
                    return(-1);
        case -5:    
        case -6:    errorlog(ERRLOG_FATAL,"Invalid action string\n");
                    return(-1);
        default:    errorlog(ERRLOG_FATAL,"Invalid parameters\n");
                    return(-1);
    }

    /* Get Ports */
    GetDlgItemText(hWnd,IDC_PORT,buf,sizeof(buf));
    for (r=0,portn=0,p=buf;;){
        if (!(q=strchr(p,' '))){
            if (strlen(p))
                r=parse_opt_port(gddc,p,&portn);
            break;
        }
        *q='\0';
        if ((r=parse_opt_port(gddc,p,&portn))!=0) break;
        p=q+1;
    }
    if (r==-1){
        errorlog(ERRLOG_FATAL,"Invalid port specification\n");
        return(-1);
    }
    if (portn==0){
        errorlog(ERRLOG_WARNING,"Default port (1-1023) is selected\n");
        gddc->port[0].pst=1;
        gddc->port[0].ped=1023;
        gddc->port[1].pst=ENDOFPORT;
        gddc->port[1].ped=ENDOFPORT;
    }else{
        gddc->port[portn].pst=ENDOFPORT;
        gddc->port[portn].ped=ENDOFPORT;
    }

    /* Get check boxes */
    gddc->hold=IsDlgButtonChecked(hWnd,IDC_HOLDTMPLOG);
    gddc->mode&=MODE_DEBUG;
    if (IsDlgButtonChecked(hWnd,IDC_PKTDUMP)) gddc->mode|=MODE_DUMP;
    if (IsDlgButtonChecked(hWnd,IDC_SECUREMODE)) gddc->mode|=MODE_SECURE;
    
    /* NIC list */
    r=SendMessage(GetDlgItem(hWnd,IDC_INTERFACE),CB_GETCURSEL,0,0);
    if (r==CB_ERR){
        errorlog(ERRLOG_FATAL,"Invalid NIC specification\n");
        return(-1);
    }
    i=SendMessage(GetDlgItem(hWnd,IDC_INTERFACE),CB_GETLBTEXTLEN,(WPARAM)r,0);
    if (i==CB_ERR || i>=MAXLEN_NICNAME){
        errorlog(ERRLOG_FATAL,"Invalid NIC specification\n");
        return(-1);
    }
    SendMessage(GetDlgItem(hWnd,IDC_INTERFACE),CB_GETLBTEXT,
        (WPARAM)r,(LPARAM)gddc->nic_name);

	if (paramdisp_flag)
	    paramdisplay_console(gddc);
    return(0);
}

/*============================================================================
 Save parameters to gdd_win32.conf file
 [INPUT]
  gddc      : Configuration data of gdd
 [RETURN]
  -1 = Error
   0 = Success
  ============================================================================
*/
int Win32MsgSubfunc_SaveCFG(GDDCONFIG *gddc)
{
    FILE        *fp;
    int         i;
    char        pt[2],buf[256];

    if ((fp=fopen(DFL_CFGFILE,"w"))==NULL) return(-1);

    fprintf(fp,"##########################################################");
    fprintf(fp,"#####################\n");
    fprintf(fp,"# Ethernet packet sniffer - GreedyDog2\n");
    fprintf(fp,"# Configuration sample\n");
    fprintf(fp,"#\n");
    fprintf(fp,"# Shadow Penguin Security http://www.shadowpenguin.org\n");
    fprintf(fp,"# Developed by UNYUN <unyun@shadowpenguin.org>\n");
    fprintf(fp,"##########################################################");
    fprintf(fp,"#####################\n");
    fprintf(fp,"\n");
    fprintf(fp,"#*** This file is generated by gddwin32 gui ***\n");
    fprintf(fp,"\n");
    fprintf(fp,"###### Port Number\n");
    for (i=0;;i++){
        if (gddc->port[i].pst==ENDOFPORT) break;
        switch(gddc->port[i].logtype){
            case LOGTYPE_BIN:      sprintf(pt,"%c",C_LOGTYPE_BIN); break;
            case LOGTYPE_TEXT:     strcpy(pt,""); break;
            case LOGTYPE_TAGTEXT:  sprintf(pt,"%c",C_LOGTYPE_TAGTEXT); break;
            case LOGTYPE_HTML:     sprintf(pt,"%c",C_LOGTYPE_HTML); break;
            case LOGTYPE_NONFILTER:sprintf(pt,"%c",C_LOGTYPE_NONFILTER); break;
            default:               strcpy(pt,""); break;
        }
        if (gddc->port[i].pst==gddc->port[i].ped)
            fprintf(fp,"port %s%d\n",pt,gddc->port[i].pst);
        else
            fprintf(fp,"port %s%d-%d\n",pt,gddc->port[i].pst,gddc->port[i].ped);
    }
    fprintf(fp,"\n");
    fprintf(fp,"###### Source IP address \n");
    fprintf(fp,"src %s\n",gddc->sourceIP);
    fprintf(fp,"\n");
    fprintf(fp,"###### Destination IP address\n");
    fprintf(fp,"dst %s\n",gddc->destIP);
    fprintf(fp,"\n");
    fprintf(fp,"###### Network interface\n");
    fprintf(fp,"nic_name %s\n",gddc->nic_name);
    fprintf(fp,"\n");
    fprintf(fp,"###### Logfile\n");
    fprintf(fp,"logfile %s\n",gddc->logfile);
    fprintf(fp,"\n");
    fprintf(fp,"###### Temporary Directory\n");
    fprintf(fp,"tmpdir %s\n",gddc->templogdir);
    fprintf(fp,"\n");
    fprintf(fp,"###### Error logfile\n");
    fprintf(fp,"error_logfile %s\n",gddc->errlog);
    fprintf(fp,"\n");
    fprintf(fp,"###### Mode\n");


    strcpy(buf,"");
    if (gddc->mode&MODE_DEBUG)  {strcat(buf,STR_MODE_DEBUG);  strcat(buf,","); }
    if (gddc->mode&MODE_DUMP)   {strcat(buf,STR_MODE_DUMP);   strcat(buf,","); }
    if (gddc->mode&MODE_SECURE) {strcat(buf,STR_MODE_SECURE); strcat(buf,","); }
    if (strlen(buf)==0)
        fprintf(fp,"#mode debug");
    else{
        if (buf[strlen(buf)-1]==',')
            buf[strlen(buf)-1]='\0';
        fprintf(fp,"mode %s\n",buf);
    }

    if (strlen(gddc->action)){
        fprintf(fp,"\n");
        fprintf(fp,"###### Action\n");
        fprintf(fp,"action %s\n",gddc->action);
    }
    fprintf(fp,"\n");
    fprintf(fp,"##### Hold temporary log\n");

    if (gddc->hold)
        fprintf(fp,"hold on\n");
    else
        fprintf(fp,"hold off\n");
    fprintf(fp,"\n");
    fclose(fp);
    return(0);
}
