/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks : hsj <hsj@shadowpenguin.org> told me the point to port
             this code to AIX.
             4lj <4lj@flow.le.to> implements "tmpfile hold" mode

 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "getopt.h"
#include "depend.h"
#include "param.h"
#include "gddconfig.h"
#include "parse.h"
#include "gen.h"
#include "defaultnic.h"
#include "gdd_printf.h"

/* Option (command line) */
static char     *optstring="p:d:s:N:D:l:t:e:a:m:f:H:hvx";

/* Option (configuration file) */
static char *cfgstring[]={
    "port",
    "dst",
    "src",
    "nic_name",
    "nit_dev",
    "logfile",
    "tmpdir",
    "error_logfile",
    "action",
    "mode",
    "hold", 
    "disable_startupmsg",
    NULL
};
enum{
    CO_PRT,
    CO_DST,
    CO_SRC,
    CO_NIC,
    CO_NIT,
    CO_LOG,
    CO_TMP,
    CO_ERR,
    CO_ACT,
    CO_MOD,
    CO_HOLD,    
    CO_DISABLMSG,    
};


/*============================================================================
 Display usage
 [INPUT]
  none
 [RETURN]
  none
  ============================================================================
*/
void usage()
{
    printf(GDD_BANNER,GDD_VERSION);
    printf("Usage: gdd [options]\n\n");
    printf("Options:\n");
    printf("  -h               : print this help message\n");
    printf("  -v               : show version\n");
    printf("  -x               : disable startup message\n");
    printf("  -H <on/off>      : hold temporary log\n");
    printf("  -p <port>        : destination and source port\n");
    printf("  -d <IPaddress>   : destination IP address\n");
    printf("  -s <IPaddress>   : source IP address\n");
    printf("  -N <nic_name>    : nic_name\n");
    printf("  -D <nit_dev>     : nit_device\n");
    printf("  -l <log_file>    : logfile\n");
    printf("  -t <tmp_logdir>  : temporary log directory\n");
    printf("  -e <err_logfile> : error logfile\n");
    printf("  -a <action>      : set action\n");
    printf("  -m <mode>        : set mode\n");
    printf("  -f <config_file> : read configuration file\n");
    printf("\n");
    printf("* If you don't specify any options, gdd reads configuration\n");
    printf("  written in \"%s\".\n",DFL_CFGFILE);
    printf("* If configuration file is read by \"-f\" option, all ");
    printf("options\n  which are specified at commandline are ignord.\n");
    printf("\n");
}

/*============================================================================
 Set default value to GDD configuration
 [INPUT]
  gddc                  : Configuration of gdd
 [RETURN]
  none
  ============================================================================
*/
void set_defaultvalue(GDDCONFIG *gddc)
{
#ifndef DFL_NIC_NAME
    char    *p;
#endif

    memset((void *)gddc,0,sizeof(GDDCONFIG));
    strcpy(gddc->logfile,   DFL_LOGFILE);
    strcpy(gddc->templogdir,DFL_TEMPLOGDIR);
    strcpy(gddc->sourceIP,  DFL_SOURCEIP);
    strcpy(gddc->destIP,    DFL_DESTIP);
#ifdef DFL_NIT_DEV
    strcpy(gddc->nit_dev,   DFL_NIT_DEV);
#else
    strcpy(gddc->nit_dev,   "");
#endif
    strcpy(gddc->errlog,    DFL_ERRLOG);
    gddc->port[0].pst=1;
    gddc->port[0].ped=1023;
    gddc->port[0].logtype=LOGTYPE_TEXT;
    gddc->port[1].pst=ENDOFPORT;
    gddc->port[1].ped=ENDOFPORT;
    gddc->mode=0x00000000;
    gddc->hold=DFL_TEMPHOLD;
    gddc->enable_libpcapfilter=0;
    gddc->startup_message=1;
    gddc->param_mode=PARAMMODE_CMDLINE;
    
#ifdef DFL_NIC_NAME
    strcpy(gddc->nic_name,DFL_NIC_NAME);
#else
    if ((p=getdefaultnic())==NULL){
        fprintf(stderr,"ERROR : Can not get default NIC.\n");
        return;
    }
    if (strlen(p)+strlen(NICDEVDIR)+1 >= sizeof(gddc->nic_name)){
        fprintf(stderr,"ERROR : Too long default NIC name.\n");
        return;
    }
    sprintf(gddc->nic_name,"%s%s",NICDEVDIR,getdefaultnic());
#endif
}   

/*============================================================================
 Display stored parameters
 [INPUT]
  gddc                  : Configuration of gdd
 [RETURN]
  none
  ============================================================================
*/
void paramdisplay_fp(GDDCONFIG *gddc,int flag)
{
    int         i;
    int         (*pprintf)(char *,...);
    

    if (! gddc->startup_message)
        return;

    if (flag==0)
        pprintf=(void *)printf;
    else
        pprintf=gdd_printf;

    printf(GDD_BANNER,GDD_VERSION);
    pprintf("[Settings]\n");
    pprintf("Dest IP address    : %s\n",gddc->destIP);
    pprintf("Source IP address  : %s\n",gddc->sourceIP);
#ifdef PCAPON
    pprintf("nic_name           : %s (libpcap)\n",gddc->nic_name);
#else
    pprintf("nic_name           : %s\n",gddc->nic_name);
#endif
    pprintf("nit_dev            : %s\n",gddc->nit_dev);
    pprintf("Logfile            : %s\n",gddc->logfile);
    pprintf("Temp log directory : %s\n",gddc->templogdir);
    pprintf("Error logfile      : %s\n",gddc->errlog);
    pprintf("Hold temporary log : %s\n",gddc->hold? "on":"off");
    pprintf("Mode               : ");
    if (gddc->mode==0x00000000)
        pprintf("[normal]\n");
    else{
        if (gddc->mode&MODE_DEBUG)  pprintf("[debug]");
        if (gddc->mode&MODE_DUMP)   pprintf("[dump]");
        if (gddc->mode&MODE_SECURE) pprintf("[secure]");
        pprintf("\n");
    }
    pprintf("Target port :\n");
    for (i=0;i<65536;i++){
        if (gddc->port[i].pst==ENDOFPORT) break;
        switch(gddc->port[i].logtype){
            case LOGTYPE_BIN:
                pprintf("%-6s ",STR_LOGTYPE_BIN);
                break;
            case LOGTYPE_TEXT:
                pprintf("%-6s ",STR_LOGTYPE_TEXT);
                break;
            case LOGTYPE_TAGTEXT:
                pprintf("%-6s ",STR_LOGTYPE_TAGTEXT);
                break;
            case LOGTYPE_HTML:
                pprintf("%-6s ",STR_LOGTYPE_HTML);
                break;
            case LOGTYPE_NONFILTER:
                pprintf("%-6s ",STR_LOGTYPE_NONFILTER);
                break;
            default:
                pprintf("????  ");
                break;
        }
        if (gddc->port[i].pst==gddc->port[i].ped)
            pprintf("%d(tcp)\n",gddc->port[i].pst);
        else
            pprintf("%d(tcp)-%d(tcp)\n",
                gddc->port[i].pst,gddc->port[i].ped);
    }
    pprintf("Action :\n");
    if (strlen(gddc->action))
        pprintf("%s\n",gddc->action);
    else
        pprintf("(none)\n");
}
void paramdisplay(GDDCONFIG *gddc)
{
    paramdisplay_fp(gddc,0);
}
void paramdisplay_console(GDDCONFIG *gddc)
{
    paramdisplay_fp(gddc,1);
}

/*============================================================================
 Check stored paramters
 [INPUT]
  gddc                  : Configuration of gdd
 [RETURN]
  -1            : error
   0            : OK
  ============================================================================
*/
int paramcheck(GDDCONFIG *gddc)
{
    static char *pp[2];
    int         i,j,dotc,slashc,r;
    char        *p;
    FILE        *fp;

    pp[0]=gddc->destIP;
    pp[1]=gddc->sourceIP;

    /* check IP addresses */
    for (i=0;i<2;i++){
        dotc=slashc=0;
        for (j=0;j<(int)strlen(pp[i]);j++){
            if (pp[i][j]=='.') dotc++;
            else if (pp[i][j]=='/') slashc++;
            else if (!isdigit((int)pp[i][j])){
                fprintf(stderr,"Invalid IP address\n");
                return(-1);
            }
        }
        if (dotc!=3 || slashc!=1){
            fprintf(stderr,"Invalid IP address\n");
            return(-1);
        }
    }

    /* check directory for logfile */
    if ((r=iswriteablefile(gddc->logfile))){
        fprintf(stderr,"Invalid logfile name or permission(%d)\n",r);
        return(-2);
    }
    if ((r=iswriteablefile(gddc->errlog))){
        fprintf(stderr,"Invalid errlogfile name or permission(%d)\n",r);
        return(-3);
    }

    /* check templog directory */
    if ((r=checkdir(gddc->templogdir))){
        fprintf(stderr,"Invalid tempdir or permission(%d)\n",r);
        return(-4);
    }
        
    /* check action */
    if (strlen(gddc->action)){
        if (!(p=strchr(gddc->action,':'))){
            fprintf(stderr,"Invalid action\n");
            return(-5);
        }
        if (*(p+1)=='\0'){
            fprintf(stderr,"Action pattern is not specified\n");
            return(-5);
        }
        if ((fp=fopen(EGREP,"r"))==NULL){
            fprintf(stderr,"%s is not found on this system\n",EGREP);
            return(-6);
        }else
            fclose(fp);
    }
    
    /* check templog hold flag */
    i=gddc->hold;
    if (i==-1){
        fprintf(stderr, "Invalid hold parameter\n");
        return(-7);
    }   
    return(0);
}

/*============================================================================
 Check whether this string contains non-digit character
 [INPUT]
  val       : input string
 [RETURN]
  0             : string is digit
 -1             : not digit
  ============================================================================
*/
int isdigitstr(char *val)
{
    int         i;
    for (i=0;i<(int)strlen(val);i++)
        if (!isdigit((int)val[i])) return(0);
    return(-1);
}

/*============================================================================
 Parse port option string
 [INPUT]
  gddc          : Configuration of gdd
  optarg        : Option value
  portn         : Count of port option specification
 [RETURN]
  -1            : Error
   0            : OK
  ============================================================================
*/
int parse_opt_port(GDDCONFIG *gddc,char *optarg,int *portn)
{
    char            ports[512],*pports,*p;
    unsigned int    i,c;

    for (i=c=0;i<strlen(optarg);i++)
        if (optarg[i]==' ' || optarg[i]=='\t') c++;
    if (c==strlen(optarg)) return(0);

    /* Store optarg to localbuffer */
    strncpy(ports,optarg,sizeof(ports)-1);
    ports[sizeof(ports)-1]='\0';
    pports=ports;

    /* Check first character of port  */
    if (!isdigit((int)*pports)) pports++;

    /* Range or single ? */
    if ((p=strchr(ports,'-'))==NULL){

        /* Single */
        if (!isdigitstr(pports)){
            fprintf(stderr,"Invalid port number\n");
            return(-1);
        }
        gddc->port[*portn].pst=atoi(pports);
        gddc->port[*portn].ped=atoi(pports);
    }else{

        /* Range */
        *p='\0';
        if (!isdigitstr(pports)
         || !isdigitstr(p+1)){
            fprintf(stderr,"Invalid port number\n");
            return(-1);
        }
        gddc->port[*portn].pst=atoi(pports);
        gddc->port[*portn].ped=atoi(p+1);
    }

    /* Get logtype from first character of port specification */
    switch(ports[0]){
        case C_LOGTYPE_NONFILTER:
            gddc->port[*portn].logtype=LOGTYPE_NONFILTER;
            break;
        case C_LOGTYPE_BIN:
            gddc->port[*portn].logtype=LOGTYPE_BIN;
            break;
        case C_LOGTYPE_TAGTEXT:
            gddc->port[*portn].logtype=LOGTYPE_TAGTEXT;
            break;
        case C_LOGTYPE_HTML:
            gddc->port[*portn].logtype=LOGTYPE_HTML;
            break;
        case C_LOGTYPE_TEXT:
            gddc->port[*portn].logtype=LOGTYPE_TEXT;
            break;
        default:
            if (isdigit((int)ports[0]))
                gddc->port[*portn].logtype=LOGTYPE_TEXT;
            else{
            fprintf(stderr,"Invalid Logtype\n");
                return(-1);
            }
    }
    *portn=*portn+1;
    return(0);
}

/*============================================================================
 Parse command
 [INPUT]
  gddc          : Configuration of gdd
  optn          : Option to handle
  optarg        : Option value
  portn         : Count of port option specification
  portsp_flag   : set 1 if port is set
 [RETURN]
  -1            : Unknown Option 
  -2            : Version query 
  -3            : Help
  -4            : Invalid portnumber
  -5            : Invalid mode
   0            : Others
  ============================================================================
*/
int parse_command(GDDCONFIG *gddc,int optn,char *optarg,
                  int *portn,int *portsp_flag)
{
    int parse_configfile(GDDCONFIG *gddc,char *cfgfile);
    
    switch(optn){
        case 'v':
            printf(GDD_BANNER,GDD_VERSION);
            return(-2);
        case 'h':
            usage();
            return(-3);
        case 'x':
        case CO_DISABLMSG:
            gddc->startup_message=0;
            break;
        case 'H':
        case CO_HOLD:
            if (!strncmp(optarg,"on",2))                
                gddc->hold=1;
            else if (!strncmp(optarg,"off",2))
                gddc->hold=0;
            else
                gddc->hold=-1;
            break;
        case 'p':
        case CO_PRT:
            if (parse_opt_port(gddc,optarg,portn))
                return(-4);
            else
                *portsp_flag=1;
            break;
        case 'd':
        case CO_DST:
            strncpy(gddc->destIP,optarg,sizeof(gddc->destIP)-1);
            gddc->destIP[sizeof(gddc->destIP)-1]='\0';
            break;
        case 's':
        case CO_SRC:
            strncpy(gddc->sourceIP,optarg,sizeof(gddc->sourceIP)-1);
            gddc->destIP[sizeof(gddc->sourceIP)-1]='\0';
            break;
        case 'N':
        case CO_NIC:
            strncpy(gddc->nic_name,optarg,sizeof(gddc->nic_name)-1);
            gddc->nic_name[sizeof(gddc->nic_name)-1]='\0';
            break;
        case 'D':
        case CO_NIT:
            strncpy(gddc->nit_dev,optarg,sizeof(gddc->nit_dev)-1);
            gddc->nit_dev[sizeof(gddc->nit_dev)-1]='\0';
            break;
        case 'l':
        case CO_LOG:
            strncpy(gddc->logfile,optarg,sizeof(gddc->logfile)-1);
            gddc->logfile[sizeof(gddc->logfile)-1]='\0';
            break;
        case 't':
        case CO_TMP:
            strncpy(gddc->templogdir,optarg,
                    sizeof(gddc->templogdir)-1);
            gddc->templogdir[sizeof(gddc->templogdir)-1]='\0';
            break;
        case 'e':
        case CO_ERR:
            strncpy(gddc->errlog,optarg,sizeof(gddc->errlog)-1);
            gddc->errlog[sizeof(gddc->errlog)-1]='\0';
            break;
        case 'a':
        case CO_ACT:
            strncpy(gddc->action,optarg,sizeof(gddc->action)-1);
            gddc->action[sizeof(gddc->action)-1]='\0';
            break;
        case 'm':
        case CO_MOD:
            gddc->mode=0x00000000;
            if (strstr(optarg,STR_MODE_DEBUG))
                gddc->mode|=MODE_DEBUG;
            if (strstr(optarg,STR_MODE_DUMP))
                gddc->mode|=MODE_DUMP;
            if (strstr(optarg,STR_MODE_SECURE))
                gddc->mode|=MODE_SECURE;
            break;
        case 'f':
            set_defaultvalue(gddc);
            return(parse_configfile(gddc,optarg));
        default:
            return(-1);
    }
    return(0);
}

/*============================================================================
 Parse configuration file
 [INPUT]
  gddc                  : Configuration of gdd
  cfgfile       : Configuration file
 [RETURN]
  -1            : Error
   0            : OK
  ============================================================================
*/
int parse_configfile(GDDCONFIG *gddc,char *cfgfile)
{
    FILE        *fp;
    int         r,i,optn,line,findflag;
    int         portn=0,portsp_flag=0;
    static char buf[4096],*optarg;
    char        def[256];

    printf("reading '%s'...\n",cfgfile);
    if ((fp=fopen(cfgfile,"r"))==NULL){
        fprintf(stderr,"Can not open configuration file '%s'\n",
                cfgfile);
        return(-1);
    }
    memset(def,0,sizeof(def));
    for (line=1;;line++){
        if (feof(fp)) break;

        /* Get line from config file */
        if (!fgets(buf,sizeof(buf)-1,fp)) break;
        buf[sizeof(buf)-1]='\0';

        /* Comment or NULL line? */
        if (buf[0]=='\0' || buf[0]=='\r' ||
            buf[0]=='\n' || buf[0]=='#') continue;
        for (i=0;i<(int)strlen(buf);i++)
            if (buf[i]!='\t' || buf[i]!=' ') break;
        if (i==(int)strlen(buf)) continue;

        /* Option is written in line? */
        for (findflag=i=0;;i++){
            if (cfgstring[i]==NULL) break;
            if (!strncmp(buf,cfgstring[i],strlen(cfgstring[i]))){
                findflag=1;
                break;  
            }
        }
        if (!findflag){
            fprintf(stderr,"Syntax error in '%s'. line %d.\n",
                cfgfile,line);
            fclose(fp);
            return(-1);
        }
        optn=i;
            
        /* Get optarg */
        if ((optarg=strchr(buf,' '))==NULL)
            optarg=strchr(buf,'\t');
        if (optarg==NULL){
            fprintf(stderr,"Parameter error in '%s'. line %d.\n",
                cfgfile,line);
            fclose(fp);
            return(-1);
        }
        if (!isdigit((int)optarg[strlen(optarg)-1]))
            optarg[strlen(optarg)-1]='\0';
            
        for (i=0;i<(int)strlen(optarg);i++)
            if (optarg[i]!=' ' &&  optarg[i]!='\t') break;
        if (i==(int)strlen(optarg)){
            fprintf(stderr,"Parameter error in '%s'. line %d.\n",
                cfgfile,line);
            fclose(fp);
            return(-1);
        }
        optarg+=i;

        /* Is this option handled yet? */
        if (def[optn] && optn!=CO_PRT){
            fprintf(stderr,"Option '%s' is already defined.\n",
                cfgstring[optn]);
            fclose(fp);
            return(-1);
        }

        /* Option handling */
        if ((r=parse_command(gddc,optn,optarg,&portn,&portsp_flag))){
            if (r==-1){
                fprintf(stderr,"Unknown option(%d) in '%s'.\n",
                    i,cfgfile);
                fclose(fp);
                return(-1);
            }
        }

        /* Define checkin */
        if (optn<sizeof(def)) def[optn]=(char)1;
        else{
            fprintf(stderr,"Internal Error : Too many options\n");
            fclose(fp);
            return(-1);
        }
    }
    fclose(fp);

    if (portsp_flag){
        gddc->port[portn].pst=ENDOFPORT;
        gddc->port[portn].ped=ENDOFPORT;
    }

    if (paramcheck(gddc)) return(-1);   
    /* paramdisplay(gddc); */
    return(0);
}

/*============================================================================
 Parse command line string
 [INPUT]
  gddc                  : Configuration of gdd
  argc          : argc from main()
  argv          : argv from main()
 [RETURN]
  -1            : Error
   0            : OK
  ============================================================================
*/
int parse_commandline(GDDCONFIG *gddc,int argc,char *argv[])
{

    int         c;
    int         r;
    int         portn=0,portsp_flag=0;

    /* Check user ID */
    if (getuid()!=0){
        fprintf(stderr,"root privilege is required.\n");
        return(-1);
    }

    /* Set default value */
    set_defaultvalue(gddc);

    if (argc==1){
        gddc->param_mode=PARAMMODE_CFGFILE;
        return(parse_configfile(gddc,DFL_CFGFILE));
    }
    
    /* Parse commandline */
    while ((c=getopt(argc,argv,optstring))!= EOF){
        if ((r=parse_command(gddc,(int)c,optarg,&portn,&portsp_flag))){
            if (r==-1){
                fprintf(stderr,"Unknown option\n");
                usage();
            }
            return(-1);
        }
    }
    if (portsp_flag){
        gddc->port[portn].pst=ENDOFPORT;
        gddc->port[portn].ped=ENDOFPORT;
    }

    if (paramcheck(gddc)) return(-1);   
    paramdisplay(gddc);

    return(0);
}
