/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    Thanks : Using libpcap was suggested by reverse
 *****************************************************************************/

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "depend.h"
#include "param.h"
#include "promisc.h"
#include "analyze.h"
#include "gddconfig.h"
#include "error.h"

#include "analyze.h"

/*============================================================================
 pcap_loop handler
 [INPUT]
  gddc          : Configuration data of gdd
 [RETURN]
  none
  ============================================================================
*/
void getpacket(unsigned char *user,struct pcap_pkthdr *h,unsigned char *p)
{
    GDDCONFIG              *gddc;

    gddc=(GDDCONFIG *)user;
    if (packet_analysis(gddc,p,h->len)){
        release_connection_table();
        exit(1);
    }
}


/*============================================================================
 Set host filter
 [INPUT]
  gddc          : Configuration data of gdd
 [RETURN]
  none
  ============================================================================
*/
char *hostfilter(GDDCONFIG *gddc,char *ipadrbuf)
{
    unsigned int            maskval,maskptn_s,maskptn_e,host_s,host_e;
    static char             buf[1024];
    char                    ipbuf[1024];
    char                    *p;

    if (strlen(ipadrbuf)>=sizeof(ipbuf)) return(NULL);
    if (strlen(ipadrbuf)==0) return(NULL);
    strcpy(ipbuf,ipadrbuf);

    if (!(p=strchr(ipbuf,'/'))) return(NULL);
    *p='\0';
    maskval=atoi(p+1);
    if (maskval==0) return(NULL);

    maskptn_s=((unsigned long)pow(2.0,(double)maskval)-1);
    maskptn_e=~maskptn_s;
    host_s=inet_addr(ipbuf)&maskptn_s;
    host_e=host_s|maskptn_e;

    sprintf(buf,"((ip[12:4]>=0x%lx and ip[12:4]<=0x%lx) or "\
                 "(ip[16:4]>=0x%lx and ip[16:4]<=0x%lx)) and",
                 (unsigned long)htonl(host_s),
                 (unsigned long)htonl(host_e),
                 (unsigned long)htonl(host_s),
                 (unsigned long)htonl(host_e));

    return(buf);
}

/*============================================================================
 Make filter string for libpcap
 [INPUT]
  gddc          : Configuration data of gdd
 [RETURN]
  none
  ============================================================================
*/
char *makefilterstring(GDDCONFIG *gddc)
{
    static char             fltstr[4096];
    char                    buf[1024];
    int                     i;
    char                    *p;

    /* Set default filter */
    strcpy(fltstr,"tcp and "\
                  "ip[0]&0xf0==0x40 and "\
                  "ip[0]&0x0f>=5 and "\
                  "ip[2:2]>=40 and "\
                  "tcp[12]/16>=5 and ");

    /* Set host filter */
    if ((p=hostfilter(gddc,gddc->sourceIP))!=NULL)
        strcat(fltstr,p);
    if ((p=hostfilter(gddc,gddc->destIP))!=NULL)
        strcat(fltstr,p);

    /* Make port filter */
    for (i=0;i<65536;i++){
        if (gddc->port[i].pst==ENDOFPORT) break;
        if (gddc->port[i].pst != gddc->port[i].ped){
            sprintf(buf,"((tcp[0:2]>=%d and tcp[0:2]<=%d)"\
                        "or"\
                        "(tcp[2:2]>=%d and tcp[2:2]<=%d))",
                gddc->port[i].pst,gddc->port[i].ped,
                gddc->port[i].pst,gddc->port[i].ped);
        }else
            sprintf(buf,"(port %d)",gddc->port[i].pst);

        if (strlen(fltstr)+strlen(buf)>=sizeof(buf)-1)
            return(NULL);
        strcat(fltstr,buf);
        if (gddc->port[i+1].pst!=ENDOFPORT){
            if (strlen(fltstr)+4>=sizeof(buf)-1)
                return(NULL);
            strcat(fltstr," or ");
        }
    }

    /* printf("fltstr='%s'\n",fltstr); */
    return(fltstr);
}

/*============================================================================
 Main loop (read packet and analyze it)
 [INPUT]
  gddc          : Configuration data of gdd
 [RETURN]
  none
  ============================================================================
*/
void sniffloop(GDDCONFIG *gddc)
{
    pcap_t                  *pd;
    struct bpf_program      fcode; 
    uint32_t                localnet,netmask;
    char                    ebuf[PCAP_ERRBUF_SIZE];
    char                    *fltstr;

    if (init_connection_table()){
        errorlog(ERRLOG_FATAL,"Can not initialize contable\n");
        return;
    }
    if (!(pd=pcap_open_live(gddc->nic_name,CHUNKSIZE,
                            1,PCAP_TIMEOUT,ebuf))){
        errorlog(ERRLOG_FATAL,"pcap_open_live: %s\n",ebuf);
        release_connection_table();
        return;
    }
    if (pcap_lookupnet(gddc->nic_name,&localnet,&netmask,ebuf)<0){
        errorlog(ERRLOG_FATAL,"pcap_lookupnet: %s\n",ebuf);
        release_connection_table();
        return;
    }
    if ((fltstr=makefilterstring(gddc))==NULL){
        errorlog(ERRLOG_FATAL,"Too long filter string.\n");
        release_connection_table();
        return;
    }
    if ( pcap_compile(pd,&fcode,fltstr,0,netmask)>=0
      && pcap_setfilter(pd,&fcode)>=0){
        gddc->enable_libpcapfilter=1;
        if (gddc->startup_message)
            fprintf(stderr,"libpcap filter enabled.\n");
    }else
        fprintf(stderr,"Can not enable libpcap filter.\n");

    if (pcap_loop(pd,-1,(void *)getpacket,(void *)gddc)<0){
        errorlog(ERRLOG_FATAL,"pcap_loop: %s\n",pcap_geterr(pd));
        release_connection_table();
        return;
    }
    pcap_close(pd);
}
